#!/bin/bash
# Script to start MDM

set -e
set -x

# Set default path for cert file if no advanced path is specified
if [[ -z ${CERT_FILE} ]]; then
    CERT_FILE="/tmp/geneva_mdm/mdm-cert.pem"
fi
# Set default path for key file if no advanced path is specified
if [[ -z ${KEY_FILE} ]]; then
    KEY_FILE="/tmp/geneva_mdm/mdm-key.pem"
fi

MDM_TMP_DIR="/tmp/MetricsExtensionData"

# In order to start MDM because we don't trust it to clean up its own stuff
# we must delete the cache of old configs/thumbprint json files from /tmp
function clean_mdm_tmp_dir {
    rm -Rf ${MDM_TMP_DIR}
}

# Extract provided .pfx file into cert and key components
# Use paths provided by CERT_FILE and KEY_FILE
function extract_pfx_file_to_cert_key_paths {
    if ! openssl pkcs12 -in ${PFX_FILE} -out ${CERT_FILE} -nodes -nokeys -passin pass: ; then
        return 1
    fi

    if ! openssl pkcs12 -in ${PFX_FILE} -out ${KEY_FILE} -nodes -nocerts -passin pass: ; then
        return 1
    fi

    return 0
}

# Splits colon separated string (parameter #1) into an global array of a given
# name (parameter #2)
function split_colon_string {
    local string=$1
    local array_name=$2

    local oifs=$IFS
    IFS=':'
    read -ra $array_name <<<"$string"
    IFS=$oifs
}

# Verify validity of cert file
function check_cert {
    local cert_file="$1"

    if ! openssl x509 -in "${cert_file}" -text -noout > /dev/null; then
        return 1
    fi

    return 0
}

# Verify validity of key file
function check_key {
    local key_file="$1"

    if ! openssl rsa -in "${key_file}" -check > /dev/null; then
        return 1
    fi

    return 0
}

# Verify openssl md5 sum of cert and key match
function check_cert_key_match {
    local cert_file="$1"
    local key_file="$2"

    local cert_md5=$(openssl x509 -noout -modulus -in ${cert_file} | openssl md5 | awk {' print $2'})
    local key_md5=$(openssl rsa -noout -modulus -in ${key_file} | openssl md5 | awk {' print $2'})

    if [[ "${cert_md5}" != "${key_md5}" ]]; then
        echo "ERROR: Cert file openssl MD5: ${cert_md5}"
        echo "ERROR: Key file openssl MD5: ${key_md5}"
        return 1
    fi

    return 0
}

# Parse all expected ENV variables and make sure they're good before we try and start
function parse_arguments {
    if ! [[ -z ${USE_DSMS_CERT} ]]; then
	    echo "INFO: USE_DSMS_CERT is set"
        USE_DSMS_CERT_PARAM="-UseDsmsCertificate"
    fi

    # If the user passes in a PFX file then we extract and split it into CERT and KEY files before
    # we continue
    if ! [[ -z ${PFX_FILE} ]]; then
        if ! extract_pfx_file_to_cert_key_paths; then
            echo "ERROR: Could not extract Cert and Key PEM files from provided PFX file"
            exit 1
        fi
    fi

    split_colon_string "$CERT_FILE" CERT_FILE_ARRAY

    for CF in "${CERT_FILE_ARRAY[@]}"; do
        # Check if the cert file exists
        if [ ! -f "${CF}" ]; then
			if [[ -z ${USE_DSMS_CERT} ]]; then
				echo "ERROR: Could not find MDM Cert file: ${CF}"
                echo "ERROR: Please make sure the file is shared with the container properly"
			    exit 1
			else
			    echo "WARNING: Could not find MDM Cert file: ${CF}"
                echo "WARNING: Please make sure the file is shared with the container properly"
                echo "WARNING: Continue with Dsms certificates only"
			    CERT_ERROR="Y"
			fi
        elif ! check_cert "${CF}"; then
			if [[ -z ${USE_DSMS_CERT} ]]; then
			    echo "ERROR: Your MDM Cert file: ${CF} is unparsable via openssl"
                echo "ERROR: Please make sure the Cert file is valid"
			    exit 1
			else
			    echo "WARNING: Your MDM Cert file: ${CF} is unparsable via openssl"
                echo "WARNING: Please make sure the Cert file is valid"
                echo "WARNING: Continue with Dsms certificates only"
			    CERT_ERROR="Y"
			fi
        fi
    done

    split_colon_string "$KEY_FILE" KEY_FILE_ARRAY

    for KF in "${KEY_FILE_ARRAY[@]}"; do
        # Check if the key file exists
        if [ ! -f "${KF}" ]; then
			if [[ -z ${USE_DSMS_CERT} ]]; then
                echo "ERROR: Could not find MDM Key file: ${KF}"
                echo "ERROR: Please make sure the file is shared with the container properly"
			    exit 1
			else
                echo "WARNING: Could not find MDM Key file: ${KF}"
                echo "WARNING: Please make sure the file is shared with the container properly"
                echo "WARNING: Continue with Dsms certificates only"
			    CERT_ERROR="Y"
			fi
        elif ! check_key "${KF}"; then
			if [[ -z ${USE_DSMS_CERT} ]]; then
                echo "ERROR: Your MDM Key file: ${KF} is unparsable via openssl"
                echo "ERROR: Please make sure the Key file is valid"
			    exit 1
			else
                echo "WARNING: Your MDM Key file: ${KF} is unparsable via openssl"
                echo "WARNING: Please make sure the Key file is valid"
                echo "WARNING: Continue with Dsms certificates only"
			    CERT_ERROR="Y"
			fi
        fi
    done

    CERT_FILE_LEN="${#CERT_FILE_ARRAY[*]}"
    KEY_FILE_LEN="${#KEY_FILE_ARRAY[*]}"

    if [ "$CERT_FILE_LEN" != "$KEY_FILE_LEN" ]; then
        echo "ERROR: $CERT_FILE_LEN certificates given, and $KEY_FILE_LEN keys given"
        echo "ERROR: Number of certificates must match number of keys"
        exit 1
    fi

    for I in $(seq 0 $(($CERT_FILE_LEN - 1))); do
        CF="${CERT_FILE_ARRAY[$I]}"
        KF="${KEY_FILE_ARRAY[$I]}"
        if ! check_cert_key_match "$CF" "$KF"; then
            echo "ERROR: Your MDM Cert file #$I: ${CERT_FILE} and your MDM Key file #$I: ${KEY_FILE} don't match"
            echo "ERROR: The openssl MD5 sum is not the same on both certs"
            exit 1
        fi
    done

    # Check to make sure METRIC_ENDPOINT is set via ENV variable
    if [[ -z "${METRIC_ENDPOINT}" ]]; then
        echo "INFO: METRIC_ENDPOINT is not set, using global default"
        FRONTEND_URL_OPTION=''
    elif [ "${METRIC_ENDPOINT}" = https://az-int.metrics.nsatc.net/ ] ||
        [ "${METRIC_ENDPOINT}" = https://global.metrics.azure.microsoft.scloud/ ] ||
        [ "${METRIC_ENDPOINT}" = https://global.metrics.azure.eaglex.ic.gov/ ]; then
        echo "INFO: METRIC_ENDPOINT is set, using ${METRIC_ENDPOINT}"
        FRONTEND_URL_OPTION="-FrontEndUrl ${METRIC_ENDPOINT}"
    elif [ "${METRIC_ENDPOINT}" = https://global.metrics.nsatc.net/ ]; then
        FRONTEND_URL_OPTION=''
        echo "WARNING: You're don't need to specify default global configuration endpoint"
        echo "Action item: Please remove METRIC_ENDPOINT=https://global.metrics.nsatc.net/ from your container configuration"
        echo "In future versions this will cause an error on startup!"
    else
        echo "ERROR: METRIC_ENDPOINT is not valid!"
        echo "ERROR: You passed in value: ${METRIC_ENDPOINT}"
        echo "The only supported value is https://az-int.metrics.nsatc.net/, and it should be only used for accessing int configuration environment"
        echo
        echo "Temporarily, for testing purposes in airgapped clouds, these can be also specified:"
        echo "* for USSec = https://global.metrics.azure.microsoft.scloud/"
        echo "* for USNat = https://global.metrics.azure.eaglex.ic.gov/"
        echo
        echo "If you've used any other option pointing to a specific stamp in the past, please remove it - it should work with default prod configuration environment"
        exit 1
    fi

    # Check to make sure CONFIG_OVERRIDES is set via ENV variable
    if [[ -z "${CONFIG_OVERRIDES}" ]]; then
        echo "INFO: CONFIG_OVERRIDES is not set"
        echo "INFO: Setting CONFIG_OVERRIDES to empty"
        CONFIG_OVERRIDES="{}"
    fi
}

# In order to start MDM because we don't trust it to clean up its own stuff
# we must delete the cache of old configs/thumbprint json files from /tmp
clean_mdm_tmp_dir

# Parse all expected ENV variables and make sure they're good before we try and start
parse_arguments

echo "Starting MDM with the following Parameters"
if ! [[ -z ${PFX_FILE} ]]; then
    echo "PFX_FILE: ${PFX_FILE}"
fi
echo "CERT_FILE: ${CERT_FILE}"
echo "KEY_FILE: ${KEY_FILE}"
echo "METRIC_ENDPOINT: ${METRIC_ENDPOINT}"
echo "CONFIG_OVERRIDES: ${CONFIG_OVERRIDES}"

# Check if transform rules dir is specified
if [[ -n "${MDM_TRANSFORM_RULES_DIR}" ]]; then
    TRANSFORM_RULES_OPTION="-TransformRulesdir ${MDM_TRANSFORM_RULES_DIR}"
    echo "TRANSFORM_RULES_OPTION: ${TRANSFORM_RULES_OPTION}"
fi

echo "Starting Metrics Extension"

if [[ -z "${CERT_ERROR}" ]]; then
    exec /usr/sbin/MetricsExtension \
        ${FRONTEND_URL_OPTION} \
        ${TRANSFORM_RULES_OPTION} \
        -Logger Console \
        -CertFile "$CERT_FILE" \
        -PrivateKeyFile "$KEY_FILE" \
        ${USE_DSMS_CERT_PARAM} \
        -ConfigOverrides "$CONFIG_OVERRIDES"
else
    exec /usr/sbin/MetricsExtension \
        ${FRONTEND_URL_OPTION} \
        ${TRANSFORM_RULES_OPTION} \
        -Logger Console \
        ${USE_DSMS_CERT_PARAM} \
        -ConfigOverrides "$CONFIG_OVERRIDES"
fi
